
#[macro_use]
extern crate serde_derive;
extern crate serde;
extern crate serde_json;

extern crate rodio;
mod base;
use base::*;
use imgui::*;
use walkdir::WalkDir;
mod support_ogl;
// mod support_glium;
use env_logger;
use log::*;

fn collect_sounds (root: &str) -> Vec<Sound> {
    WalkDir::new(root)
    .into_iter()
    .filter_map(|e| e.ok())
    .filter(|e| e.path().is_file())
    .map(|f| Sound::new(&f.path().to_string_lossy().to_string()))
    .filter_map(|e| e.ok())
    .collect()
}


fn main() {
    env_logger::init();
    let dev = rodio::default_output_device().unwrap();
    
    let mut sounds = collect_sounds("media");
    
    let mut timeline = Timeline::default();

    // return;
    let mut pattern_col: usize = 0;
    let mut pattern_row: usize = 0;
    let mut active_sound_idx = 0;


    // let mut active_bar = Bar::default();
    // let mut active_bar = &mut timeline.patterns[0].bars[0];
    let mut active_pattern = Pattern::default();
    let mut active_sound: Option<Sound> = None;
    // let mut active_pattern = &mut timeline.patterns[0];


    let mut system = support_ogl::init(file!());
    let s = system.imgui.style_mut();
    s.window_rounding = 1.5;
    // let system = support_glium::init(file!());

    system.main_loop(move |_, ui| {
        Window::new(im_str!("sources"))
            // .resizable(false)
            .position([400.0, 140.0], Condition::Appearing)
            .collapsible(false)
            // .always_auto_resize(true)
            .size([300.0, 300.0], Condition::FirstUseEver)
            .build(ui, || {

                // ui.show_default_style_editor();

                ui.tree_node(im_str!("snds"))
                .opened(true, Condition::Appearing)
                .build(|| {
                    for s in &mut sounds {
                        ui.tree_node(&im_str!("{}", s.name)).build(|| {
                            // ui.same_line(0.0);
                            if ui.small_button(im_str!("add to pattern")) {
                                active_pattern.sounds[pattern_row][pattern_col] = Some(s.clone());
                            }
                            if ui.small_button(im_str!("load")) {
                                active_sound = Some(s.clone());
                            }
                            ui.same_line(0.0);
                            if ui.small_button(im_str!("play")) {
                                s.play(&dev);
                            }
                        });
                    }
                });
            });

        Window::new(im_str!("pattern"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {

                if ui.small_button(&im_str!("column++")) {
                    active_pattern.extend_length(1);
                }
                ui.same_line(0.0);
                
                if ui.small_button(&im_str!("row++")) {
                    active_pattern.extend_row(1);
                }

                ui.same_line(0.0);
                if ui.small_button(&im_str!("ply")) {
                    active_pattern.play_mut(&dev);
                }

                ui.drag_int(im_str!("repeat"), &mut active_pattern.repeat)
                .max(32)
                .speed(0.05)
                .build();
                
                ui.drag_int(im_str!("bpm"), &mut active_pattern.bpm).build();

                for row_idx in 0..active_pattern.sounds.len() {
                    ui.text(im_str!("row {}", row_idx));

                    for snd_idx in 0..active_pattern.sounds[row_idx].len() {
                        ui.same_line(0.0);
                        // ui.text(im_str!("snd {}", snd_idx));

                        let mut active = false;
                        if let Some(snd) = &active_pattern.sounds[row_idx][snd_idx] {
                            if snd.active {
                                active = true;
                            }
                        }

                        // let style = ui.push_style_color(StyleColor::Button, [0.0, 1.0, 0.1, 1.0]);
                        // style.pop(ui);
                  

                        let mut label = match &active_pattern.sounds[row_idx][snd_idx] {
                            Some(_s) => "X",
                            None => " "
                        };

                        if active {
                            label = "*";
                        }

                        if ui.small_button(&im_str!("{}##{}{}", label, row_idx, snd_idx)) {
                            pattern_col = snd_idx;
                            pattern_row = row_idx;
                            println!("r{}c{}", row_idx, snd_idx);
                            active_pattern.sounds[row_idx][snd_idx] = active_sound.clone();
                        }

                      
                     

                    }



                }

                ui.text(im_str!("pat {} {}", pattern_col, pattern_row));


        
            });
    
            Window::new(im_str!("sound"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {
                // let snd = &active_bar.sounds[pattern_col];
                if let Some(s) = &mut active_sound {
                    ui.text(im_str!("name   {}", s.name));
                    // let _ = DragFloat::new()
                    ui.drag_float(im_str!("volume"), &mut s.volume)
                        .min(0.0)
                        .max(1.0)
                        .speed(0.01)
                        .build();
                    ui.drag_float(im_str!("speed"), &mut s.speed)
                        .min(0.0)
                        .max(2.0)
                        .speed(0.01)
                        .build();
                    ui.drag_int(im_str!("roll"), &mut s.roll)
                        .max(32)
                        .speed(0.05)
                        .build();

                
                    if ui.small_button(&im_str!("play")) {
                        s.play(&dev);
                    }
                
                }
                if ui.small_button(&im_str!("clear")) {
                    active_sound = None;
                }
            });
    
            Window::new(im_str!("tracks"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {
               
                if ui.small_button(&im_str!("add")) {
                    active_pattern.play_mut(&dev);
                }
            });
    
    
        });
}
